import uuid

from fastapi import Response
from fastapi.datastructures import QueryParams
from sqlmodel import select

from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.user import User
from openec2.db.image import AMI
from openec2.api.describe_images import DescribeImagesResponse, ImagesSet, Image


def describe_images(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
    _: User,
):
    images: list[Image] = []
    for ami in db.exec(select(AMI)).all():
        images.append(
            Image(
                imageId=ami.id,
                imageState="available",
                name=ami.originalFilename,
            ),
        )

    return Response(
        DescribeImagesResponse(
            requestId=uuid.uuid4().hex,
            imagesSet=ImagesSet(
                items=images,
            ),
        ).to_xml(),
        media_type="application/xml",
    )
