from typing import cast
from pathlib import Path
from urllib.parse import urlparse
import uuid
import shutil

from fastapi import HTTPException
from fastapi.datastructures import QueryParams
import requests

from openec2.config import OpenEC2Config, ConfigSingleton
from openec2.db import DatabaseDep
from openec2.db.user import User
from openec2.db.image import AMI


def import_image(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
    user: User,
):
    first_disk_image_url = params["DiskContainer.1.Url"]
    url = urlparse(first_disk_image_url)
    ami_id = uuid.uuid4().hex

    imageLocation = cast(Path, config.images)
    imageLocation.mkdir(exist_ok=True)
    dst = imageLocation / ami_id
    filename = Path(url.path).name
    match url.scheme:
        case "https":
            with dst.open("wb") as f:
                with requests.get(first_disk_image_url, stream=True) as r:
                    r.raise_for_status()
                    for chunk in r.iter_content(8196):
                        f.write(chunk)
        case "file":
            if not ConfigSingleton.of().config.debug:
                raise HTTPException(
                    status_code=400,
                    detail="Unsupported scheme",
                )
            shutil.copy(
                url.path,
                str(dst),
            )
        case _:
            raise Exception("Unknown scheme")

    db.add(
        AMI(
            id=ami_id,
            description=None,
            originalFilename=filename,
            owner_id=user.id,
        ),
    )
    db.commit()
    return ami_id
