from typing import Generator, Annotated
from fastapi import Depends
from sqlmodel import Session, create_engine

from openec2.config import ConfigSingleton

engine = create_engine(
    ConfigSingleton.of().config.database.url,
    connect_args={
        "check_same_thread": False,
    },
    echo=ConfigSingleton.of().config.database.debug,
)


def get_session() -> Generator[Session]:
    with Session(engine) as session:
        yield session


DatabaseDep = Annotated[Session, Depends(get_session)]
