import random

from sqlmodel import select

from openec2.db import DatabaseDep
from openec2.db.instance import Instance


def ipv4_to_int(ip: str) -> int:
    i = 0
    for idx, p in enumerate(ip.split(".")):
        i += int(p) << (3 - idx) * 8
    return i


def int_to_ipv4(ip: int) -> str:
    parts: list[int] = []
    for i in reversed(range(4)):
        parts.append(
            (ip >> i * 8) & 255,
        )
    return ".".join(str(p) for p in parts)


def generate_mac() -> str:
    mac_bytes = random.randbytes(6)
    mac = ""
    for idx, b in enumerate(mac_bytes):
        # Ensure we have a unicast MAC
        if idx == 0:
            b = b & (255 - 1)

        h = hex(b)[2:]
        if len(h) == 1:
            mac += f"0{h}:"
        else:
            mac += f"{h}:"
    return mac[:-1]


def generate_available_mac(db: DatabaseDep) -> str:
    mac = ""
    while True:
        mac = generate_mac()
        if (
            db.exec(select(Instance).where(Instance.interfaceMac == mac)).first()
            is None
        ):
            break
    return mac
